@@ -76,11 +76,10 @@ class ExpandDims(Function, Module):
         A = self.parent_tensors[0]
         return xp.squeeze(grad_output, axis=self.axis) if A.requires_grad else None
 
-class Concatenate(Function, Module):
+class Concatenate(Function):
     name = "Concatenate"
     def __init__(self, axis):
         Function.__init__(self)
-        Module.__init__(self)
         self.axis = axis
     def forward(self, *inputs: xp.ndarray) -> xp.ndarray:
         return xp.concatenate(inputs, axis=self.axis)
@@ -117,23 +116,11 @@ class Slice(Function, Module):
 
 
 class Cast(Function):
-    """
-    Cast tensor to a different dtype while maintaining autograd graph.
-
-    Optional memsave mode:
-    - When memsave=True, this op may drop the parent's data buffer after
-      producing the casted output to avoid holding two full copies.
-    - This is only safe when the input tensor will not be used again later
-      in the forward pass and when the input is not a leaf/parameter.
-    - Backward for Cast does not need the parent's data, only its dtype,
-      so gradients remain correct.
-    """
     name = "Cast"
-    def __init__(self, target_dtype, memsave: bool = False):
+    def __init__(self, target_dtype):
         super().__init__()
         self.target_dtype = target_dtype
         self.original_dtype = None
-        self.memsave = bool(memsave)
     def forward(self, input_data: xp.ndarray) -> xp.ndarray:
         self.original_dtype = input_data.dtype
         if self.original_dtype == self.target_dtype:
@@ -143,17 +130,11 @@ class Cast(Function):
         x = self.parent_tensors[0]
         if not x.requires_grad:
             return None
+        if grad_output is None:
+            return None
         if grad_output.dtype == self.original_dtype:
             return grad_output
         return grad_output.astype(self.original_dtype, copy=False)
-    # Best-effort memory saver: drop parent's data after successful cast when safe.
-    def _memsave(self, parent_tensors: "Sequence[Tensor]", output_tensor: "Tensor"):
-        # Only consider when dtype actually changes and parent is non-leaf
-        if self.original_dtype != self.target_dtype and getattr(parent_tensors[0], "grad_fn", None) is not None:
-            # Release the parent's data buffer; backward for Cast doesn't need it.
-            del parent_tensors[0].data
-            ng.flush(gc=False)
-            parent_tensors[0].data = output_tensor.data
 
 class Pad(Function, Module):
     name = "Pad"
@@ -214,6 +195,7 @@ class Clone(Function, Module):
         return grad_output if A.requires_grad else None
 
 
+
 class SlidingWindowView(Function, Module):
     """
     Smart Vectorized Sliding Window View with AutoDiff Support and
@@ -236,7 +218,7 @@ class SlidingWindowView(Function, Module):
         self.slices = tuple(slices)
         return xp.lib.stride_tricks.sliding_window_view(
             A, self.window_shape, self.axes)[self.slices]
-    def backward(self, grad_output):
+    def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         A = self.parent_tensors[0]
         grad_buffer = xp.zeros(A.shape, dtype=grad_output.dtype)
         kwargs = {"writeable": True} if xp is np else {}
@@ -246,7 +228,7 @@ class SlidingWindowView(Function, Module):
         # Accumulate gradients using cached view
         grad_view += grad_output
         return grad_buffer
-
+    
 
 
 def reshape(A, new_shape):
@@ -259,9 +241,8 @@ def expand_dims(A, axis):
     return ExpandDims(axis)(A)
 def concat(tensors: Sequence["Tensor"], axis: int) -> "Tensor":
     return Concatenate(axis=axis)(*tensors)
-def cast(A, target_dtype, memsave: bool = False):
-    """Functional cast with optional memsave to avoid dual copies."""
-    return Cast(target_dtype, memsave=memsave)(A)
+def cast(A, target_dtype):
+    return Cast(target_dtype)(A)
 def pad(A, pad_width, mode='constant', constant_values=0, memsave=False, **kwargs):
     return Pad(pad_width, mode, constant_values, memsave=memsave, **kwargs)(A)
 def sliding_window_view(A, window_shape: Sequence[int], axes: Union[int, Tuple[int, ...]] = (2, 3), 
@@ -270,5 +251,6 @@ def sliding_window_view(A, window_shape: Sequence[int], axes: Union[int, Tuple[i
 def clone(A):
     return Clone()(A)
 
+
 # newaxis constant for numpy-style indexing
 newaxis = None
